@@ -79,7 +79,7 @@ NNS.ARMA.optim <- function(variable,
   
   if(any(class(variable)%in%c("tbl","data.table"))) variable <- as.vector(unlist(variable))
   
-  if(sum(is.na(variable)) > 0) stop("You have some missing values, please address.")
+  if(anyNA(variable)) stop("You have some missing values, please address.")
   
   n <- length(variable)
   
@@ -98,9 +98,10 @@ NNS.ARMA.optim <- function(variable,
     h_oos <- NULL
   }
   
-  if(is.null(training.set)) training.set <- .8 * n
+  if(is.null(training.set)) training.set <- floor(.8 * n)
+  training.set <- as.integer(training.set)
   
-  h_eval <- h_is <- n - training.set
+  h_eval <- h_is <- as.integer(n - training.set)
   
   actual <- tail(variable, h_eval)
   
@@ -146,6 +147,7 @@ NNS.ARMA.optim <- function(variable,
         )
         doParallel::registerDoParallel(cl)
         invisible(data.table::setDTthreads(1))  # Restrict threading for parallelization
+        parallel::clusterEvalQ(cl, library(NNS))
       } else {
         foreach::registerDoSEQ()
         invisible(data.table::setDTthreads(0))  # Default threading
@@ -484,7 +486,7 @@ NNS.ARMA.optim <- function(variable,
     
     lfp <- length(final.predicted)
     
-    starting.point <- min(training.set, min(n - lfp))
+    starting.point <- as.integer(n - lfp)
     
     lines((starting.point + 1) : (starting.point + lfp), final.predicted, col = "red", lwd = 2, lty = 2)
     
@@ -510,10 +512,7 @@ NNS.ARMA.optim <- function(variable,
       legend("topleft", legend = c("Variable", "Internal Validation", "Forecast"), 
              col = c("steelblue", "red", "red"), lty = c(1, 2, 1), bty = "n", lwd = 2)
     } 
-    
-    
-    
-    
+   
   }
   
   
