@@ -1,4 +1,4 @@
-NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL, n.best = NULL, type = NULL, point.est = NULL, point.only = FALSE,
+NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, n.best = NULL, type = NULL, point.est = NULL, point.only = FALSE,
                        plot = FALSE, residual.plot = TRUE, location = NULL, noise.reduction = 'off', dist = "L2",
                        return.values = FALSE, plot.regions = FALSE, ncores = NULL, confidence.interval = NULL){
   
@@ -35,21 +35,14 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
   
   original.matrix <- cbind.data.frame(original.DV, original.IVs)
   norm.matrix <- apply(original.matrix, 2, function(z) NNS.rescale(z, 0, 1))
-  
+
   minimums <- apply(original.IVs, 2, min)
   maximums <- apply(original.IVs, 2, max)
-  
-  dep_a <- tryCatch(NNS.copula(original.matrix), error = function(e) .5^n)
-  dep_b <- tryCatch(NNS.copula(norm.matrix), error = function(e) .5^n)
-  dep_c <- tryCatch(mean(unlist(cor(original.matrix)[-1,1])), error = function(e) .5^n)
-
-  dependence <- stats::fivenum(c(dep_a, dep_b, dep_c))[4]
-  
-  if(is.null(order)) order <- max(1, ifelse(dependence*10 %% 1 < .5, floor(dependence * 10), ceiling(dependence * 10)))
 
+ 
   ###  Regression Point Matrix
-  if(is.numeric(order)){
-    reg.points <- lapply(1:ncol(original.IVs), function(b) NNS.reg(original.IVs[, b], original.DV, factor.2.dummy = factor.2.dummy, order = order, stn = stn, type = type, noise.reduction = noise.reduction, plot = FALSE, multivariate.call = TRUE, ncores = 1)$x)
+  if(is.numeric(order) || is.null(order)){
+    reg.points <- lapply(1:ncol(original.IVs), function(b) NNS.reg(original.IVs[, b], original.DV, factor.2.dummy = factor.2.dummy, order = order, type = type, noise.reduction = noise.reduction, plot = FALSE, multivariate.call = TRUE, ncores = 1)$x)
     
     if(length(unique(sapply(reg.points, length))) != 1){
       reg.points.matrix <- do.call(cbind, lapply(reg.points, `length<-`, max(lengths(reg.points))))
@@ -60,9 +53,10 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
     reg.points.matrix <- original.IVs
   }
   
-
+  
   ### If regression points are error (not likely)...
   if(length(reg.points.matrix[ , 1]) == 0  || is.null(reg.points.matrix)){
+    stn <- .95
     for(i in 1 : n){
       part.map <- NNS.part(original.IVs[ , i], original.DV, order = order, type = type, noise.reduction = noise.reduction, obs.req = 0)
       dep <- NNS.dep(original.IVs[ , i], original.DV)$Dependence
@@ -153,11 +147,11 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
   
   fitted.matrix <- data.table::data.table(original.IVs, y = original.DV, y.hat, mean.by.id.matrix[ , .(NNS.ID)])
   
-  fitted.matrix$residuals <- fitted.matrix$y - fitted.matrix$y.hat
+  fitted.matrix$residuals <- fitted.matrix$y.hat - fitted.matrix$y
   fitted.matrix[, bias := gravity(residuals),  by = NNS.ID]
   fitted.matrix$y.hat <- fitted.matrix$y.hat - fitted.matrix$bias
   fitted.matrix$bias <- NULL
-  
+
   
   data.table::setkey(mean.by.id.matrix, 'NNS.ID')
   REGRESSION.POINT.MATRIX <- mean.by.id.matrix[ , c("obs") := NULL]
@@ -169,7 +163,10 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
   
   data.table::setnames(REGRESSION.POINT.MATRIX, 1:n, colnames(mean.by.id.matrix)[1:n])
   
-  if(is.null(n.best)) n.best <- max(1, floor((1-dependence)*sqrt(n)))
+  if(is.null(n.best)){
+    dependence <- NNS.copula(cbind(original.IVs, original.DV))
+    n.best <- max(1, floor((1-dependence)*sqrt(n)))
+  }
 
   if(n.best > 1 && !point.only){
     if(num_cores > 1){
@@ -186,12 +183,11 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
     
     if(!is.null(type)) y.hat <- ifelse(y.hat %% 1 < 0.5, floor(y.hat), ceiling(y.hat))
   }
-  
+ 
   
   
   ### Point Estimates
   if (!is.null(point.est)) {
-    
     # Calculate central points
     central.points <- apply(REGRESSION.POINT.MATRIX[, .SD, .SDcols = 1:n], 2, gravity)
     
@@ -381,16 +377,15 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
   }
   
   rhs.partitions <- data.table::data.table(reg.points.matrix)
-  fitted.matrix$residuals <-  original.DV - fitted.matrix$y.hat
+  fitted.matrix$residuals <-   fitted.matrix$y.hat - original.DV
   
   if(!is.null(type) && type=="class"){
     R2 <- as.numeric(format(mean(fitted.matrix$y.hat==fitted.matrix$y), digits = 4))
   } else {
     y.mean <- mean(fitted.matrix$y)
     R2 <- (sum((fitted.matrix$y - y.mean)*(fitted.matrix$y.hat - y.mean))^2)/(sum((fitted.matrix$y - y.mean)^2)*sum((fitted.matrix$y.hat - y.mean)^2))
-    
   }
-  
+
   
   lower.pred.int <- NULL
   upper.pred.int <- NULL
@@ -398,10 +393,10 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
   
   if(is.numeric(confidence.interval)){
     fitted.matrix[, `:=` ( 'conf.int.pos' = abs(UPM.VaR((1-confidence.interval)/2, degree = 1, residuals)) + y.hat)]
-    fitted.matrix[, `:=` ( 'conf.int.neg' = y.hat - abs(LPM.VaR((1-confidence.interval)/2, degree = 1, residuals)))]
+    fitted.matrix[, `:=` ( 'conf.int.neg' = y.hat - abs(UPM.VaR((1-confidence.interval)/2, degree = 1, residuals)))]
     
     if(!is.null(point.est)){
-      lower.pred.int = predict.fit - abs(LPM.VaR((1-confidence.interval)/2, degree = 1, fitted.matrix$residuals))
+      lower.pred.int = predict.fit - abs(UPM.VaR((1-confidence.interval)/2, degree = 1, fitted.matrix$residuals))
       upper.pred.int = abs(UPM.VaR((1-confidence.interval)/2, degree = 1, fitted.matrix$residuals)) + predict.fit
       
       pred.int = data.table::data.table(lower.pred.int, upper.pred.int)
@@ -464,7 +459,7 @@ NNS.M.reg <- function (X_n, Y, factor.2.dummy = TRUE, order = NULL, stn = NULL,
     r2.leg <- bquote(bold(R ^ 2 == .(format(R2, digits = 4))))
     if(!is.null(type) && type=="class") r2.leg <- paste("Accuracy: ", R2) 
     plot(seq_along(original.DV), original.DV, pch = 1, lwd = 2, col = "steelblue", xlab = "Index", ylab = expression(paste("y (blue)   ", hat(y), " (red)")), cex.lab = 1.5, mgp = c(2, .5, 0))
-    lines(seq_along(y.hat), y.hat, col = 'red', lwd = 2, lty = 1)
+    lines(seq_along(fitted.matrix$y.hat), fitted.matrix$y.hat, col = 'red', lwd = 2, lty = 1)
     
     if(is.numeric(confidence.interval)){
       polygon(c(seq_along(y.hat), rev(seq_along(y.hat))), c(na.omit(fitted.matrix$conf.int.pos), rev(na.omit(fitted.matrix$conf.int.neg))), 
